package org.my.algorithm;

import java.util.Dictionary;
import java.util.Iterator;

import org.cishell.framework.CIShellContext;
import org.cishell.framework.algorithm.Algorithm;
import org.cishell.framework.algorithm.AlgorithmExecutionException;
import org.cishell.framework.data.BasicData;
import org.cishell.framework.data.Data;
import org.cishell.framework.data.DataProperty;
import org.osgi.service.log.LogService;

import prefuse.data.Table;

public class HelloMeFromFile implements Algorithm {
    private Data[] data;
    private Dictionary parameters;
    private CIShellContext ciShellContext;
	private String nameColumn;
	private String favorNumberColumn;
	private LogService logger;
    
    public HelloMeFromFile(Data[] data,
    				  Dictionary parameters,
    				  CIShellContext ciShellContext) {
        this.data = data;
        this.parameters = parameters;
        this.ciShellContext = ciShellContext;
        this.logger = (LogService) ciShellContext.getService(LogService.class.getName());

		nameColumn = (String) parameters.get("name");
		favorNumberColumn = (String) parameters.get("favor_number");
    }

    public Data[] execute() throws AlgorithmExecutionException {
    	Table inputTable = (Table) data[0].getData();
    	Iterator<?> rowsIterator = inputTable.iterator();
    	int nameColumnIndex = inputTable.getColumnNumber(nameColumn);
    	int favorNumberColumnIndex = inputTable.getColumnNumber(favorNumberColumn);
    	
		while (rowsIterator.hasNext()) {
			int currentRowNumber = Integer.parseInt(rowsIterator.next().toString());
			String name = inputTable.get(currentRowNumber, nameColumnIndex).toString();
			Integer favorNumber;
			try {
				favorNumber = Integer.valueOf(inputTable.get(currentRowNumber, favorNumberColumnIndex).toString());
			} catch (NumberFormatException e) {
				this.logger.log(LogService.LOG_WARNING, "Warning");
				this.logger.log(LogService.LOG_DEBUG, "Debug");
				this.logger.log(LogService.LOG_ERROR, "Error");
				throw new AlgorithmExecutionException("Favor number column must hold only integer value!", e);
			} 
			
			this.logger.log(LogService.LOG_INFO, name);
			this.logger.log(LogService.LOG_INFO, favorNumber.toString());
		}
		
    	Table table = generateOutputTable();
    	
        return generateOutData(table);
    }
    
    /* Create an output table */
    private Table generateOutputTable() {
    	Table table = new Table();
    	table.addColumn("name", String.class);
    	table.addColumn("Favor number square root", Integer.class);
    	int rowNumber = table.addRow();
    	table.set(rowNumber, "name", "haha");
    	table.set(rowNumber, "Favor number square root", 100);
    	rowNumber = table.addRow();
    	table.set(rowNumber, "name", "haha");
    	table.set(rowNumber, "Favor number square root", 200);
    	
        return table;
    }
    
    private Data[] generateOutData(Table table) {
		
		/*
		 * After getting the output in table format make it available to the user.
		 */
		Data output = new BasicData(table, Table.class.getName());
		Dictionary<String, Object> metadata = output.getMetadata();
		metadata.put(DataProperty.LABEL, "output001");
		metadata.put(DataProperty.PARENT, this.data[0]);
		metadata.put(DataProperty.TYPE, DataProperty.TABLE_TYPE);
		
		return new Data[] { output };
    }
}